from autonomous import log
from autonomous.tasks import AutoTasks
from tasks import mocktask, longmocktask, parametermocktask, errormocktask
import time
from celery.result import AsyncResult
import pytest


def test_create_task(app):
    at = AutoTasks()
    assert at._connection
    assert at.queue
    task_id = at.task(mocktask)
    time.sleep(2)
    assert at.get_task(task_id)
    assert at.get_status(task_id)
    assert at.get_result(task_id)
    assert at.get_all()


def test_param_task(app):
    at = AutoTasks()
    assert at.connection
    assert at.queue
    task_id = at.task(parametermocktask, 1, 2, "hello", key="value")
    time.sleep(2)
    assert at.get_task(task_id)
    assert at.get_status(task_id)
    assert at.get_result(task_id) == 3
    assert at.get_all()


def test_error_task(app):
    at = AutoTasks()
    task_id = at.task(errormocktask)
    time.sleep(2)
    assert at.get_status(task_id) == "FAILURE"
    try:
        response = at.get_result(task_id)
    except Exception as e:
        log(e)
    else:
        pytest.fail(f"Exception not raised: {response}")


def test_base_long_task(app):
    at = AutoTasks()
    assert at.connection
    assert at.queue
    task_id = at.task(longmocktask)
    time.sleep(2)
    assert at.get_task(task_id)
    assert at.get_status(task_id)
    assert at.get_result(task_id) == "success"
    assert at.get_all()



import time
from autonomous import log
from rq.decorators import job
from autonomous.tasks import AutoTasks

rq_app = AutoTasks()


@job
def mocktask():
    time.sleep(1)
    log("MockTask")
    return "success"


@job
def longmocktask():
    time.sleep(30)
    return "success"


@job
def parametermocktask(*args, **kwargs):
    log("ParameterMockTask", args, kwargs)

    return args[0] + args[1]


@job
def errormocktask():
    raise Exception("ErrorMockTask")



import redis
from rq import Queue
import os


class AutoTasks:
    _connection = None
    queue = None

    def __init__(self, connection_string="redis://redis:6379"):
        if not AutoTasks._connection:
            AutoTasks._connection = redis.from_url(os.environ.get("RQ_DEFAULT_CONNECTION", connection_string))
            AutoTasks.queue = Queue(connection=self._connection)

    def task(self, job, *args, **kwargs):
        task = self.queue.enqueue(job, *args, **kwargs)
        return task.id

    # get job given its id
    def get_task(self, job_id):
        try:
            return self.queue.fetch_job(job_id)
        except Exception as e:
            return f"invalid:\t{e}"

    # get job given its id
    def get_status(self, job_id):
        try:
            return self.queue.fetch_job(job_id).get_status()
        except Exception as e:
            return f"invalid:\t{e}"

    # get job given its id
    def get_result(self, job_id):
        try:
            return self.queue.fetch_job(job_id).result
        except Exception as e:
            return f"invalid:\t{e}"

    # get all jobs
    def get_all(self):
        # init all_jobs list
        all_jobs = list(
            set(
                [
                    self.queue.started_job_registry.get_job_ids(),
                    self.queue.job_ids.get_job_ids(),  # queued job ids
                    self.queue.failed_job_registry.get_job_ids(),
                    self.queue.deferred_job_registry.get_job_ids(),
                    self.queue.finished_job_registry.get_job_ids(),
                    self.queue.scheduled_job_registry.get_job_ids(),
                ]
            )
        )
        # iterate over job ids list and fetch jobs
        for job_id in all_jobs:
            all_jobs.append(self.get_task(job_id))
        return all_jobs
